/****************************************************************/
/*                                                              */
/*  This example demonstrates Locking of I_CACHE by ways        */
/*                                                              */
/*  (a) The instruction cache is enabled with WAY-1,2,3 locked  */
/*      and WAY=0 Open                                          */
/*  (b) Dummy calls to all the functions which need to be locked*/
/*      is made from the main code                              */
/*  (c) The WAY-0 is locked, and WAY-1,2,3 are unlocked         */
/*                                                              */
/*                                  Initial Version             */
/*                                  Kunal Singh                 */
/*                                  DSP applications, ADI       */
/*                                  Dated 3 April, 2005         */
/*                                                              */
/*                                  Re-Release                  */
/*                                  for VisualDSP++ 5.0         */
/*                                  Andreas Pellkofer           */
/*                                  GPDSP Applications, ADI     */
/*                                  Dated 9 March, 2009         */
/*                                                              */
/****************************************************************/


#include <blackfin.h>
#include <services_types.h>
#include <stdio.h>
#include "main.h"


/****************************************************/
/* Number of entries for DCPLB & ICPLB descriptors  */
/****************************************************/
#define DCPLB_CNT 10
#define ICPLB_CNT 9

 
/************************************************/
/*      Entries for the DCPLBx_ADDR             */
/************************************************/
#pragma section ("data1")

u32 D_cplb_addrs[DCPLB_CNT] =
    {
        0xff800000,
        0xff900000, 
        0x00000000,
        0x00400000,
        0x00800000, 
        0x00C00000, 
        0x01000000,
        0x01400000,
        0x01800000, 
        0x01C00000 
    };


/************************************************/
/*      Entries for the DCPLBx_DATA             */
/************************************************/
#pragma section ("data1")

u32 D_cplb_data[DCPLB_CNT] =
    {
        0x2001f,
        0x2001f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f
    };

/************************************************/
/*      Entries for the ICPLBx_ADDR             */
/************************************************/
#pragma section ("data1")

u32 I_cplb_addrs[9] =
    {
        0xFFA00000,
        0x00000000,
        0x00400000,
        0x00800000,
        0x00C00000,
        0x01000000,
        0x01400000,
        0x01800000,
        0x01C00000
    };


/************************************************/
/*      Entries for the ICPLBx_DATA             */
/************************************************/
#pragma section ("data1")

u32 I_cplb_data[9] =
    {
        0x00020003,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005
    };


/********************************/
/*  Lock Control for the ICache */
/*      1: Lock the way         */
/*      0: Don't Lock           */
/********************************/
#pragma section ("data1")

u32 Lock_Control[4] =
    {
        0,
        0,
        0,
        0
    };


/********************/
/*  Data Variables  */
/********************/
#pragma section("sdram1")
u32 data_A [ARRAY_LENGTH];


#pragma section("sdram1")
u32 data_B [ARRAY_LENGTH];


#pragma section("sdram2")
u32 Result [ARRAY_LENGTH];


void main()
{
    u32 i;
    u32 dcplb_cnt, icplb_cnt, d_bank_cnt, disable_lru;

    /********************************************/
    /* Configure Both the Data banks as Cache   */
    /********************************************/
//    d_bank_cnt = 2;
//    Config_D_Cache(D_cplb_addrs, D_cplb_data, dcplb_cnt, d_bank_cnt);

    /********************************************/
    /* Parameters to be used for Conig_I_Cache  */
    /********************************************/
    icplb_cnt = ICPLB_CNT;
    disable_lru = 0;


    /****************************/
    /*  Unlock the Way- 1,2,3   */
    /*  Keep the Way-0 Unlocked */
    /****************************/
    Lock_Control[1] = 1;
    Lock_Control[2] = 1;
    Lock_Control[3] = 1;
    Config_I_Cache(I_cplb_addrs, I_cplb_data, icplb_cnt, disable_lru, Lock_Control);


    /********************************************/
    /* Make dummy calls to all the functions    */
    /* which need to be locked in Way-0         */
    /********************************************/
    matrix_add(Result, data_A, data_B, ARRAY_LENGTH);


    /****************************/
    /*  Lock the Way-0          */
    /*  Unlock the Way- 1,2,3   */
    /****************************/
    Lock_Control[0] = 1;
    Lock_Control[1] = 0;
    Lock_Control[2] = 0;
    Lock_Control[3] = 0;
    Config_I_Cache(I_cplb_addrs, I_cplb_data, icplb_cnt, disable_lru, Lock_Control);

    printf("\n The Functions have been locked to the Way-0\n");
}
